package api

import (
	"cmf/db/model"
	"cmf/db/repo"
	"cmf/log"
	"errors"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func MatLib(r *GinRouter) {
	r.POST("/mat/create", CreateMat)
	r.POST("/mat/delete/:id", DeleteMat)
	r.GET("/mat/list", MatList)
	r.GET("/mat/detail/:id", MatDetail)
	r.POST("/mat/update", UpdateMat)
}

func CreateMat(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var data model.MatLib
	err := c.ShouldBindJSON(&data)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	data.CreateTime = time.Now()
	data.UpdateTime = time.Now()
	return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionMat, &data)
}

func DeleteMat(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionMat, _id)
}

func MatList(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	page, size, query := UtilQueryPageSize(c)
	catsConf, err := GetMatLibCategory(apictx)
	if err != nil {
		return nil, err
	}
	err = ParseCategories(query, apictx.CreateRepoCtx(), catsConf)
	if err != nil {
		return nil, err
	}

	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionMat,
		Page:        page,
		Size:        size,
		Query:       query,
		Sort:        bson.M{"createTime": -1},
	})
}

func MatDetail(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	box := &model.MatLib{}
	found, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: repo.CollectionMat,
		Query:       repo.Map{"_id": id},
	}, box)
	if err != nil {
		log.Error(err)
		return nil, err
	}

	if !found {
		return nil, errors.New("未找到该数据")
	}
	return box, nil
}

func UpdateMat(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var box model.MatLib
	err := c.ShouldBindJSON(&box)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	if box.Id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionMat, box.Id.Hex(), &box)
}
